type AccountDiscriminator = {
    kind: 'field';
    name: string;
} | {
    kind: 'size';
};
declare const fieldAccountDiscriminator: (name: string) => AccountDiscriminator;
declare const sizeAccountDiscriminator: () => AccountDiscriminator;

type AccountSeed = {
    kind: 'programId';
} | {
    kind: 'constant';
    type: TypeNode;
    value: ValueNode;
} | {
    kind: 'variable';
    name: string;
    type: TypeNode;
    docs: string[];
};
declare const programSeed: () => AccountSeed;
declare const constantSeed: (type: TypeNode, value: ValueNode) => AccountSeed;
declare const stringConstantSeed: (value: string) => AccountSeed;
declare const variableSeed: (name: string, type: TypeNode, docs?: string | string[]) => AccountSeed;
declare const publicKeySeed: (name: string, docs?: string | string[]) => AccountSeed;
declare const stringSeed: (name: string, docs?: string | string[]) => AccountSeed;

type ValueNode = ScalarValueNode | PublicKeyValueNode | ListValueNode | TupleValueNode | SetValueNode | MapValueNode | OptionValueNode | StructValueNode | EnumValueNode;
type ScalarValueNode = {
    kind: 'number';
    value: number;
} | {
    kind: 'boolean';
    value: boolean;
} | {
    kind: 'string';
    value: string;
};
declare const vScalar: (scalar: number | boolean | string) => ScalarValueNode;
type PublicKeyValueNode = {
    kind: 'publicKey';
    value: string;
};
declare const vPublicKey: (value: string) => PublicKeyValueNode;
type ListValueNode = {
    kind: 'list';
    values: ValueNode[];
};
declare const vList: (values: ValueNode[]) => ListValueNode;
type TupleValueNode = {
    kind: 'tuple';
    values: ValueNode[];
};
declare const vTuple: (values: ValueNode[]) => TupleValueNode;
type SetValueNode = {
    kind: 'set';
    values: ValueNode[];
};
declare const vSet: (values: ValueNode[]) => SetValueNode;
type MapValueNode = {
    kind: 'map';
    values: [ValueNode, ValueNode][];
};
declare const vMap: (values: [ValueNode, ValueNode][]) => MapValueNode;
type OptionValueNode = {
    kind: 'optionNone';
} | {
    kind: 'optionSome';
    value: ValueNode;
};
declare const vNone: () => OptionValueNode;
declare const vSome: (value: ValueNode) => OptionValueNode;
type StructValueNode = {
    kind: 'struct';
    values: Record<string, ValueNode>;
};
declare const vStruct: (values: Record<string, ValueNode>) => StructValueNode;
type EnumValueNode = {
    kind: 'enum';
    enumType: string;
    variant: string;
    value: StructValueNode | TupleValueNode | 'empty' | 'scalar';
    importFrom: ImportFrom | null;
};
declare const vEnum: (enumType: string, variant: string, value?: StructValueNode | TupleValueNode | 'empty' | 'scalar', importFrom?: ImportFrom | null) => EnumValueNode;

type AnchorDiscriminator = ListValueNode;
declare const getAnchorInstructionDiscriminator: (idlName: string) => AnchorDiscriminator;
declare const getAnchorAccountDiscriminator: (idlName: string) => AnchorDiscriminator;

/**
 * Tells Kinobi where a dependency is located.
 * This could be a recognized dependency, or any other string
 * as long as renderers know how to map them into real libraries.
 */
type ImportFrom = RelativeImportFrom | 'umi' | 'umiSerializers' | string;
/**
 * Tells Kinobi where a local dependency is located.
 * - `generated` means the dependency is located within the `generated` code.
 * - `hooked` means the dependency is located in the source code but outside of the `generated` code.
 *   This is typically a dedicated folder that is used internally by the auto-generated code.
 *
 * These can be interpreted by renderers as they see fit as different languages
 * have different ways of importing local files.
 */
type RelativeImportFrom = 'generated' | 'hooked';

type BytesCreatedOnChain = {
    kind: 'number';
    value: number;
    includeHeader: boolean;
} | {
    kind: 'arg';
    name: string;
    includeHeader: boolean;
} | {
    kind: 'account';
    name: string;
    importFrom: ImportFrom;
    includeHeader: boolean;
} | {
    kind: 'resolver';
    name: string;
    importFrom: ImportFrom;
};
declare const bytesFromNumber: (value: number, includeHeader?: boolean) => BytesCreatedOnChain;
declare const bytesFromArg: (arg: string, includeHeader?: boolean) => BytesCreatedOnChain;
declare const bytesFromAccount: (account: string, importFrom?: ImportFrom, includeHeader?: boolean) => BytesCreatedOnChain;
declare const bytesFromResolver: (name: string, importFrom?: ImportFrom) => BytesCreatedOnChain;

interface Visitor<T = void> {
    visitRoot: (root: RootNode) => T;
    visitProgram: (program: ProgramNode) => T;
    visitAccount: (account: AccountNode) => T;
    visitAccountData: (accountData: AccountDataNode) => T;
    visitInstruction: (instruction: InstructionNode) => T;
    visitInstructionAccount: (instructionAccount: InstructionAccountNode) => T;
    visitInstructionDataArgs: (instructionDataArgs: InstructionDataArgsNode) => T;
    visitInstructionExtraArgs: (instructionExtraArgs: InstructionExtraArgsNode) => T;
    visitDefinedType: (definedType: DefinedTypeNode) => T;
    visitError: (error: ErrorNode) => T;
    visitArrayType: (arrayType: ArrayTypeNode) => T;
    visitLinkType: (linkType: LinkTypeNode) => T;
    visitEnumType: (enumType: EnumTypeNode) => T;
    visitEnumEmptyVariantType: (enumEmptyVariantType: EnumEmptyVariantTypeNode) => T;
    visitEnumStructVariantType: (enumStructVariantType: EnumStructVariantTypeNode) => T;
    visitEnumTupleVariantType: (enumTupleVariantType: EnumTupleVariantTypeNode) => T;
    visitMapType: (mapType: MapTypeNode) => T;
    visitOptionType: (optionType: OptionTypeNode) => T;
    visitSetType: (setType: SetTypeNode) => T;
    visitStructType: (structType: StructTypeNode) => T;
    visitStructFieldType: (structFieldType: StructFieldTypeNode) => T;
    visitTupleType: (tupleType: TupleTypeNode) => T;
    visitBoolType: (boolType: BoolTypeNode) => T;
    visitBytesType: (bytesType: BytesTypeNode) => T;
    visitNumberType: (numberType: NumberTypeNode) => T;
    visitNumberWrapperType: (numberWrapperType: NumberWrapperTypeNode) => T;
    visitPublicKeyType: (publicKeyType: PublicKeyTypeNode) => T;
    visitStringType: (stringType: StringTypeNode) => T;
}
declare function visit<T>(node: Node, visitor: Visitor<T>): T;

declare class BaseThrowVisitor<T> implements Visitor<T> {
    visitRoot(root: RootNode): T;
    visitProgram(program: ProgramNode): T;
    visitAccount(account: AccountNode): T;
    visitAccountData(accountData: AccountDataNode): T;
    visitInstruction(instruction: InstructionNode): T;
    visitInstructionAccount(instructionAccount: InstructionAccountNode): T;
    visitInstructionDataArgs(instructionDataArgs: InstructionDataArgsNode): T;
    visitInstructionExtraArgs(instructionExtraArgs: InstructionExtraArgsNode): T;
    visitDefinedType(definedType: DefinedTypeNode): T;
    visitError(error: ErrorNode): T;
    visitArrayType(arrayType: ArrayTypeNode): T;
    visitLinkType(linkType: LinkTypeNode): T;
    visitEnumType(enumType: EnumTypeNode): T;
    visitEnumEmptyVariantType(enumEmptyVariantType: EnumEmptyVariantTypeNode): T;
    visitEnumStructVariantType(enumStructVariantType: EnumStructVariantTypeNode): T;
    visitEnumTupleVariantType(enumTupleVariantType: EnumTupleVariantTypeNode): T;
    visitMapType(mapType: MapTypeNode): T;
    visitOptionType(optionType: OptionTypeNode): T;
    visitSetType(setType: SetTypeNode): T;
    visitStructType(structType: StructTypeNode): T;
    visitStructFieldType(structFieldType: StructFieldTypeNode): T;
    visitTupleType(tupleType: TupleTypeNode): T;
    visitBoolType(boolType: BoolTypeNode): T;
    visitBytesType(bytesType: BytesTypeNode): T;
    visitNumberType(numberType: NumberTypeNode): T;
    visitNumberWrapperType(numberWrapperType: NumberWrapperTypeNode): T;
    visitPublicKeyType(publicKeyType: PublicKeyTypeNode): T;
    visitStringType(stringType: StringTypeNode): T;
}

declare class GetByteSizeVisitor extends BaseThrowVisitor<number | null> {
    private availableDefinedTypes;
    private visitedDefinedTypes;
    private definedTypeStack;
    registerDefinedTypes(definedTypes: DefinedTypeNode[]): void;
    visitAccount(account: AccountNode): number | null;
    visitAccountData(accountData: AccountDataNode): number | null;
    visitInstruction(instruction: InstructionNode): number | null;
    visitInstructionDataArgs(instructionDataArgs: InstructionDataArgsNode): number | null;
    visitInstructionExtraArgs(instructionExtraArgs: InstructionExtraArgsNode): number | null;
    visitDefinedType(definedType: DefinedTypeNode): number | null;
    visitArrayType(arrayType: ArrayTypeNode): number | null;
    visitLinkType(linkType: LinkTypeNode): number | null;
    visitEnumType(enumType: EnumTypeNode): number | null;
    visitEnumEmptyVariantType(): number | null;
    visitEnumStructVariantType(enumStructVariantType: EnumStructVariantTypeNode): number | null;
    visitEnumTupleVariantType(enumTupleVariantType: EnumTupleVariantTypeNode): number | null;
    visitMapType(): number | null;
    visitOptionType(optionType: OptionTypeNode): number | null;
    visitSetType(): number | null;
    visitStructType(structType: StructTypeNode): number | null;
    visitStructFieldType(structFieldType: StructFieldTypeNode): number | null;
    visitTupleType(tupleType: TupleTypeNode): number | null;
    visitBoolType(boolType: BoolTypeNode): number | null;
    visitBytesType(bytesType: BytesTypeNode): number | null;
    visitNumberType(numberType: NumberTypeNode): number | null;
    visitNumberWrapperType(numberWrapperType: NumberWrapperTypeNode): number | null;
    visitPublicKeyType(): number | null;
    visitStringType(stringType: StringTypeNode): number | null;
    protected sumSizes(sizes: (number | null)[]): number | null;
}

declare class NodeStack {
    private readonly stack;
    constructor(stack?: Node[]);
    push(node: Node): void;
    pop(): Node | undefined;
    peek(): Node | undefined;
    all(): Node[];
    isEmpty(): boolean;
    clone(): NodeStack;
    toString(): string;
    toStringArray(): string[];
    matchesWithNames(names: string[]): boolean;
}

declare const LOG_LEVELS: readonly ["debug", "trace", "info", "warn", "error"];
type LogLevel = typeof LOG_LEVELS[number];
declare const logError: (message: string, hint?: string) => void;
declare const logWarn: (message: string, hint?: string) => void;
declare const logInfo: (message: string, hint?: string) => void;
declare const logTrace: (message: string, hint?: string) => void;
declare const logDebug: (message: string, debug?: any, hint?: string) => void;
declare const getLevelIndex: (level: LogLevel) => number;
declare const shouldLogLevel: (level: LogLevel) => boolean;

type ValidatorItem = {
    message: string;
    level: LogLevel;
    node: Node;
    stack: NodeStack;
};
declare class ValidatorBag {
    readonly items: ValidatorItem[];
    constructor(items?: ValidatorItem[]);
    add(item: ValidatorItem): ValidatorBag;
    mergeWith(others: ValidatorBag[]): ValidatorBag;
    error(message: string, node: Node, stack: NodeStack): ValidatorBag;
    warn(message: string, node: Node, stack: NodeStack): ValidatorBag;
    info(message: string, node: Node, stack: NodeStack): ValidatorBag;
    trace(message: string, node: Node, stack: NodeStack): ValidatorBag;
    debug(message: string, node: Node, stack: NodeStack): ValidatorBag;
    orderByLevel(): ValidatorBag;
    log(): void;
}

declare class GetDefaultValidatorBagVisitor implements Visitor<ValidatorBag> {
    protected stack: NodeStack;
    protected program: ProgramNode | null;
    protected definedTypes: Set<string>;
    visitRoot(root: RootNode): ValidatorBag;
    visitProgram(program: ProgramNode): ValidatorBag;
    visitAccount(account: AccountNode): ValidatorBag;
    visitAccountData(accountData: AccountDataNode): ValidatorBag;
    visitInstruction(instruction: InstructionNode): ValidatorBag;
    visitInstructionAccount(instructionAccount: InstructionAccountNode): ValidatorBag;
    visitInstructionDataArgs(instructionDataArgs: InstructionDataArgsNode): ValidatorBag;
    visitInstructionExtraArgs(instructionExtraArgs: InstructionExtraArgsNode): ValidatorBag;
    visitDefinedType(definedType: DefinedTypeNode): ValidatorBag;
    visitError(error: ErrorNode): ValidatorBag;
    visitArrayType(arrayType: ArrayTypeNode): ValidatorBag;
    visitLinkType(linkType: LinkTypeNode): ValidatorBag;
    visitEnumType(enumType: EnumTypeNode): ValidatorBag;
    visitEnumEmptyVariantType(enumEmptyVariantType: EnumEmptyVariantTypeNode): ValidatorBag;
    visitEnumStructVariantType(enumStructVariantType: EnumStructVariantTypeNode): ValidatorBag;
    visitEnumTupleVariantType(enumTupleVariantType: EnumTupleVariantTypeNode): ValidatorBag;
    visitMapType(mapType: MapTypeNode): ValidatorBag;
    visitOptionType(optionType: OptionTypeNode): ValidatorBag;
    visitSetType(setType: SetTypeNode): ValidatorBag;
    visitStructType(structType: StructTypeNode): ValidatorBag;
    visitStructFieldType(structFieldType: StructFieldTypeNode): ValidatorBag;
    visitTupleType(tupleType: TupleTypeNode): ValidatorBag;
    visitBoolType(boolType: BoolTypeNode): ValidatorBag;
    visitBytesType(bytesType: BytesTypeNode): ValidatorBag;
    visitNumberType(numberType: NumberTypeNode): ValidatorBag;
    visitNumberWrapperType(numberWrapperType: NumberWrapperTypeNode): ValidatorBag;
    visitPublicKeyType(publicKeyType: PublicKeyTypeNode): ValidatorBag;
    visitStringType(stringType: StringTypeNode): ValidatorBag;
    protected pushNode(node: Node): void;
    protected popNode(): Node | undefined;
}

type DefinedTypeHistogram = {
    [key: string]: {
        total: number;
        inAccounts: number;
        inDefinedTypes: number;
        inInstructionArgs: number;
        directlyAsInstructionArgs: number;
    };
};
declare class GetDefinedTypeHistogramVisitor implements Visitor<DefinedTypeHistogram> {
    private mode;
    private stackLevel;
    visitRoot(root: RootNode): DefinedTypeHistogram;
    visitProgram(program: ProgramNode): DefinedTypeHistogram;
    visitAccount(account: AccountNode): DefinedTypeHistogram;
    visitAccountData(accountData: AccountDataNode): DefinedTypeHistogram;
    visitInstruction(instruction: InstructionNode): DefinedTypeHistogram;
    visitInstructionAccount(): DefinedTypeHistogram;
    visitInstructionDataArgs(instructionDataArgs: InstructionDataArgsNode): DefinedTypeHistogram;
    visitInstructionExtraArgs(instructionExtraArgs: InstructionExtraArgsNode): DefinedTypeHistogram;
    visitDefinedType(definedType: DefinedTypeNode): DefinedTypeHistogram;
    visitError(): DefinedTypeHistogram;
    visitArrayType(arrayType: ArrayTypeNode): DefinedTypeHistogram;
    visitLinkType(linkType: LinkTypeNode): DefinedTypeHistogram;
    visitEnumType(enumType: EnumTypeNode): DefinedTypeHistogram;
    visitEnumEmptyVariantType(): DefinedTypeHistogram;
    visitEnumStructVariantType(enumStructVariantType: EnumStructVariantTypeNode): DefinedTypeHistogram;
    visitEnumTupleVariantType(enumTupleVariantType: EnumTupleVariantTypeNode): DefinedTypeHistogram;
    visitMapType(mapType: MapTypeNode): DefinedTypeHistogram;
    visitOptionType(optionType: OptionTypeNode): DefinedTypeHistogram;
    visitSetType(setType: SetTypeNode): DefinedTypeHistogram;
    visitStructType(structType: StructTypeNode): DefinedTypeHistogram;
    visitStructFieldType(structFieldType: StructFieldTypeNode): DefinedTypeHistogram;
    visitTupleType(tupleType: TupleTypeNode): DefinedTypeHistogram;
    visitBoolType(): DefinedTypeHistogram;
    visitBytesType(): DefinedTypeHistogram;
    visitNumberType(): DefinedTypeHistogram;
    visitNumberWrapperType(numberWrapperType: NumberWrapperTypeNode): DefinedTypeHistogram;
    visitPublicKeyType(): DefinedTypeHistogram;
    visitStringType(): DefinedTypeHistogram;
    protected mergeHistograms(histograms: DefinedTypeHistogram[]): DefinedTypeHistogram;
}

declare class GetNodeInlineStringVisitor implements Visitor<string> {
    visitRoot(root: RootNode): string;
    visitProgram(program: ProgramNode): string;
    visitAccount(account: AccountNode): string;
    visitAccountData(accountData: AccountDataNode): string;
    visitInstruction(instruction: InstructionNode): string;
    visitInstructionAccount(instructionAccount: InstructionAccountNode): string;
    visitInstructionDataArgs(instructionDataArgs: InstructionDataArgsNode): string;
    visitInstructionExtraArgs(instructionExtraArgs: InstructionExtraArgsNode): string;
    visitDefinedType(definedType: DefinedTypeNode): string;
    visitError(error: ErrorNode): string;
    visitArrayType(arrayType: ArrayTypeNode): string;
    visitLinkType(linkType: LinkTypeNode): string;
    visitEnumType(enumType: EnumTypeNode): string;
    visitEnumEmptyVariantType(enumEmptyVariantType: EnumEmptyVariantTypeNode): string;
    visitEnumStructVariantType(enumStructVariantType: EnumStructVariantTypeNode): string;
    visitEnumTupleVariantType(enumTupleVariantType: EnumTupleVariantTypeNode): string;
    visitMapType(mapType: MapTypeNode): string;
    visitOptionType(optionType: OptionTypeNode): string;
    visitSetType(setType: SetTypeNode): string;
    visitStructType(structType: StructTypeNode): string;
    visitStructFieldType(structFieldType: StructFieldTypeNode): string;
    visitTupleType(tupleType: TupleTypeNode): string;
    visitBoolType(boolType: BoolTypeNode): string;
    visitBytesType(bytesType: BytesTypeNode): string;
    visitNumberType(numberType: NumberTypeNode): string;
    visitNumberWrapperType(numberWrapperType: NumberWrapperTypeNode): string;
    visitPublicKeyType(): string;
    visitStringType(stringType: StringTypeNode): string;
    displayArrayLikeSize(size: ArrayTypeNode['size']): string;
}

declare class GetNodeTreeStringVisitor implements Visitor<string> {
    indent: number;
    readonly separator: string;
    constructor(separator?: string);
    visitRoot(root: RootNode): string;
    visitProgram(program: ProgramNode): string;
    visitAccount(account: AccountNode): string;
    visitAccountData(accountData: AccountDataNode): string;
    visitInstruction(instruction: InstructionNode): string;
    visitInstructionAccount(instructionAccount: InstructionAccountNode): string;
    visitInstructionDataArgs(instructionDataArgs: InstructionDataArgsNode): string;
    visitInstructionExtraArgs(instructionExtraArgs: InstructionExtraArgsNode): string;
    visitDefinedType(definedType: DefinedTypeNode): string;
    visitError(error: ErrorNode): string;
    visitArrayType(arrayType: ArrayTypeNode): string;
    visitLinkType(linkType: LinkTypeNode): string;
    visitEnumType(enumType: EnumTypeNode): string;
    visitEnumEmptyVariantType(enumEmptyVariantType: EnumEmptyVariantTypeNode): string;
    visitEnumStructVariantType(enumStructVariantType: EnumStructVariantTypeNode): string;
    visitEnumTupleVariantType(enumTupleVariantType: EnumTupleVariantTypeNode): string;
    visitMapType(mapType: MapTypeNode): string;
    visitOptionType(optionType: OptionTypeNode): string;
    visitSetType(setType: SetTypeNode): string;
    visitStructType(structType: StructTypeNode): string;
    visitStructFieldType(structFieldType: StructFieldTypeNode): string;
    visitTupleType(tupleType: TupleTypeNode): string;
    visitBoolType(boolType: BoolTypeNode): string;
    visitBytesType(bytesType: BytesTypeNode): string;
    visitNumberType(numberType: NumberTypeNode): string;
    visitNumberWrapperType(numberWrapperType: NumberWrapperTypeNode): string;
    visitPublicKeyType(): string;
    visitStringType(stringType: StringTypeNode): string;
    indented(text: string): string;
}

type InstructionInput = InstructionArg | InstructionAccount;
type InstructionArg = {
    kind: 'arg';
    name: string;
    defaultsTo: InstructionArgDefault;
};
type InstructionAccount = {
    kind: 'account';
} & Omit<InstructionAccountNode, 'kind'>;
type ResolvedInstructionInput = ResolvedInstructionAccount | ResolvedInstructionArg;
type ResolvedInstructionAccount = InstructionAccount & {
    isPda: boolean;
    dependsOn: InstructionDependency[];
    resolvedIsSigner: boolean | 'either';
    resolvedIsOptional: boolean;
};
type ResolvedInstructionArg = InstructionArg & {
    dependsOn: InstructionDependency[];
};
declare class GetResolvedInstructionInputsVisitor extends BaseThrowVisitor<ResolvedInstructionInput[]> {
    protected stack: InstructionInput[];
    protected resolved: ResolvedInstructionInput[];
    protected visitedAccounts: Map<string, ResolvedInstructionAccount>;
    protected visitedArgs: Map<string, ResolvedInstructionArg>;
    protected error: string | null;
    getError(): string | null;
    visitInstruction(instruction: InstructionNode): ResolvedInstructionInput[];
    resolveInstructionInput(instruction: InstructionNode, input: InstructionInput): void;
    resolveInstructionAccount(instruction: InstructionNode, account: InstructionAccount): ResolvedInstructionAccount;
    resolveInstructionArg(instruction: InstructionNode, arg: InstructionArg & {
        kind: 'arg';
    }): ResolvedInstructionArg;
    resolveInstructionDependencies(instruction: InstructionNode, parent: InstructionInput, dependencies: InstructionDependency[]): void;
    getInstructionDependencies(input: InstructionInput): InstructionDependency[];
}

declare class BaseNodeVisitor implements Visitor<Node> {
    visitRoot(root: RootNode): Node;
    visitProgram(program: ProgramNode): Node;
    visitAccount(account: AccountNode): Node;
    visitAccountData(accountData: AccountDataNode): Node;
    visitInstruction(instruction: InstructionNode): Node;
    visitInstructionAccount(instructionAccount: InstructionAccountNode): Node;
    visitInstructionDataArgs(instructionDataArgs: InstructionDataArgsNode): Node;
    visitInstructionExtraArgs(instructionExtraArgs: InstructionExtraArgsNode): Node;
    visitDefinedType(definedType: DefinedTypeNode): Node;
    visitError(error: ErrorNode): Node;
    visitArrayType(arrayType: ArrayTypeNode): Node;
    visitLinkType(linkType: LinkTypeNode): Node;
    visitEnumType(enumType: EnumTypeNode): Node;
    visitEnumEmptyVariantType(enumEmptyVariantType: EnumEmptyVariantTypeNode): Node;
    visitEnumStructVariantType(enumStructVariantType: EnumStructVariantTypeNode): Node;
    visitEnumTupleVariantType(enumTupleVariantType: EnumTupleVariantTypeNode): Node;
    visitMapType(mapType: MapTypeNode): Node;
    visitOptionType(optionType: OptionTypeNode): Node;
    visitSetType(setType: SetTypeNode): Node;
    visitStructType(structType: StructTypeNode): Node;
    visitStructFieldType(structFieldType: StructFieldTypeNode): Node;
    visitTupleType(tupleType: TupleTypeNode): Node;
    visitBoolType(boolType: BoolTypeNode): Node;
    visitBytesType(bytesType: BytesTypeNode): Node;
    visitNumberType(numberType: NumberTypeNode): Node;
    visitNumberWrapperType(numberWrapperType: NumberWrapperTypeNode): Node;
    visitPublicKeyType(publicKeyType: PublicKeyTypeNode): Node;
    visitStringType(stringType: StringTypeNode): Node;
}

declare class AutoSetAnchorDiscriminatorsVisitor extends BaseNodeVisitor {
    protected program: ProgramNode | null;
    visitProgram(program: ProgramNode): Node;
    visitAccount(account: AccountNode): Node;
    visitInstruction(instruction: InstructionNode): Node;
}

declare class AutoSetFixedAccountSizesVisitor extends BaseThrowVisitor<RootNode> {
    visitRoot(root: RootNode): RootNode;
}

declare class BaseNodeOrNullVisitor implements Visitor<Node | null> {
    visitRoot(root: RootNode): Node | null;
    visitProgram(program: ProgramNode): Node | null;
    visitAccount(account: AccountNode): Node | null;
    visitAccountData(accountData: AccountDataNode): Node | null;
    visitInstruction(instruction: InstructionNode): Node | null;
    visitInstructionAccount(instructionAccount: InstructionAccountNode): Node | null;
    visitInstructionDataArgs(instructionDataArgs: InstructionDataArgsNode): Node | null;
    visitInstructionExtraArgs(instructionExtraArgs: InstructionExtraArgsNode): Node | null;
    visitDefinedType(definedType: DefinedTypeNode): Node | null;
    visitError(error: ErrorNode): Node | null;
    visitArrayType(arrayType: ArrayTypeNode): Node | null;
    visitLinkType(linkType: LinkTypeNode): Node | null;
    visitEnumType(enumType: EnumTypeNode): Node | null;
    visitEnumEmptyVariantType(enumEmptyVariantType: EnumEmptyVariantTypeNode): Node | null;
    visitEnumStructVariantType(enumStructVariantType: EnumStructVariantTypeNode): Node | null;
    visitEnumTupleVariantType(enumTupleVariantType: EnumTupleVariantTypeNode): Node | null;
    visitMapType(mapType: MapTypeNode): Node | null;
    visitOptionType(optionType: OptionTypeNode): Node | null;
    visitSetType(setType: SetTypeNode): Node | null;
    visitStructType(structType: StructTypeNode): Node | null;
    visitStructFieldType(structFieldType: StructFieldTypeNode): Node | null;
    visitTupleType(tupleType: TupleTypeNode): Node | null;
    visitBoolType(boolType: BoolTypeNode): Node | null;
    visitBytesType(bytesType: BytesTypeNode): Node | null;
    visitNumberType(numberType: NumberTypeNode): Node | null;
    visitNumberWrapperType(numberWrapperType: NumberWrapperTypeNode): Node | null;
    visitPublicKeyType(publicKeyType: PublicKeyTypeNode): Node | null;
    visitStringType(stringType: StringTypeNode): Node | null;
}

type NodeSelector = {
    kind: Node['kind'] | '*';
    name?: string;
    stack?: string | string[];
    program?: string;
} | NodeSelectorFunction;
type NodeSelectorFunction = (node: Node, stack: NodeStack, program: ProgramNode | null) => boolean;
declare const toNodeSelectorFunction: (selector: NodeSelector) => NodeSelectorFunction;

type NodeTransformer<T extends Node = Node> = (node: T, stack: NodeStack, program: ProgramNode | null) => T | null;
type NodeTransform<T extends NodeSelector = NodeSelector> = {
    selector: T;
    transformer: NodeTransformer;
};
declare class TransformNodesVisitor extends BaseNodeOrNullVisitor {
    readonly transforms: NodeTransform<NodeSelectorFunction>[];
    readonly stack: NodeStack;
    protected program: ProgramNode | null;
    constructor(transforms: NodeTransform[]);
    visitRoot(root: RootNode): Node | null;
    visitProgram(program: ProgramNode): Node | null;
    visitAccount(account: AccountNode): Node | null;
    visitAccountData(accountData: AccountDataNode): Node | null;
    visitInstruction(instruction: InstructionNode): Node | null;
    visitInstructionAccount(instructionAccount: InstructionAccountNode): Node | null;
    visitInstructionDataArgs(instructionDataArgs: InstructionDataArgsNode): Node | null;
    visitInstructionExtraArgs(instructionExtraArgs: InstructionExtraArgsNode): Node | null;
    visitDefinedType(definedType: DefinedTypeNode): Node | null;
    visitError(error: ErrorNode): Node | null;
    visitArrayType(arrayType: ArrayTypeNode): Node | null;
    visitLinkType(linkType: LinkTypeNode): Node | null;
    visitEnumType(enumType: EnumTypeNode): Node | null;
    visitEnumEmptyVariantType(enumEmptyVariantType: EnumEmptyVariantTypeNode): Node | null;
    visitEnumStructVariantType(enumStructVariantType: EnumStructVariantTypeNode): Node | null;
    visitEnumTupleVariantType(enumTupleVariantType: EnumTupleVariantTypeNode): Node | null;
    visitMapType(mapType: MapTypeNode): Node | null;
    visitOptionType(optionType: OptionTypeNode): Node | null;
    visitSetType(setType: SetTypeNode): Node | null;
    visitStructType(structType: StructTypeNode): Node | null;
    visitStructFieldType(structFieldType: StructFieldTypeNode): Node | null;
    visitTupleType(tupleType: TupleTypeNode): Node | null;
    visitBoolType(boolType: BoolTypeNode): Node | null;
    visitBytesType(bytesType: BytesTypeNode): Node | null;
    visitNumberType(numberType: NumberTypeNode): Node | null;
    visitNumberWrapperType(numberWrapperType: NumberWrapperTypeNode): Node | null;
    visitPublicKeyType(publicKeyType: PublicKeyTypeNode): Node | null;
    visitStringType(stringType: StringTypeNode): Node | null;
    protected applyTransforms(node: Node | null): Node | null;
}

declare class CreateSubInstructionsFromEnumArgsVisitor extends TransformNodesVisitor {
    readonly map: Record<string, string>;
    protected allDefinedTypes: Map<string, DefinedTypeNode>;
    constructor(map: Record<string, string>);
    visitRoot(root: RootNode): Node | null;
}

declare class DeduplicateIdenticalDefinedTypesVisitor extends BaseThrowVisitor<RootNode> {
    visitRoot(root: RootNode): RootNode;
}

declare class DeleteNodesVisitor extends TransformNodesVisitor {
    constructor(selectors: NodeSelector[]);
}

declare class FlattenInstructionArgsStructVisitor extends TransformNodesVisitor {
    constructor();
}

type FlattenStructOptions = string[] | '*';
declare class FlattenStructVisitor extends TransformNodesVisitor {
    readonly map: Record<string, FlattenStructOptions>;
    constructor(map: Record<string, FlattenStructOptions>);
}
declare const flattenStruct: (node: Node, options?: FlattenStructOptions) => StructTypeNode;

declare class SetAccountDiscriminatorFromFieldVisitor extends TransformNodesVisitor {
    readonly map: Record<string, {
        field: string;
        value: ValueNode;
    }>;
    constructor(map: Record<string, {
        field: string;
        value: ValueNode;
    }>);
}

type InstructionAccountDefaultRule = InstructionAccountDefault & {
    /** The name of the instruction account or a pattern to match on it. */
    account: string | RegExp;
    /** @defaultValue Defaults to searching accounts on all instructions. */
    instruction?: string;
    /** @defaultValue `false`. */
    ignoreIfOptional?: boolean;
};
declare const DEFAULT_INSTRUCTION_ACCOUNT_DEFAULT_RULES: InstructionAccountDefaultRule[];
declare class SetInstructionAccountDefaultValuesVisitor extends BaseNodeVisitor {
    protected readonly rules: InstructionAccountDefaultRule[];
    protected allAccounts: Map<string, AccountNode>;
    constructor(rules: InstructionAccountDefaultRule[]);
    visitRoot(root: RootNode): Node;
    visitProgram(program: ProgramNode): Node;
    visitInstruction(instruction: InstructionNode): Node;
    protected matchRule(instruction: InstructionNode, account: InstructionAccountNode): InstructionAccountDefaultRule | undefined;
}

type Discriminator = {
    value: ValueNode;
    /** @defaultValue `new NumberTypeNode('u8')` */
    type?: TypeNode;
    /** @defaultValue `"discriminator"` */
    name?: string;
    /** @defaultValue `"omitted"` */
    strategy?: 'optional' | 'omitted';
    /** @defaultValue `[]` */
    docs?: string[];
};
declare class SetInstructionDiscriminatorsVisitor extends TransformNodesVisitor {
    readonly map: Record<string, Discriminator>;
    constructor(map: Record<string, Discriminator>);
}

declare class SetMissingDefinedTypesVisitor extends BaseThrowVisitor<RootNode> {
    readonly programs: ProgramNode[];
    constructor(programs: IdlInputs);
    visitRoot(root: RootNode): RootNode;
}

type NumberWrapperMap = Record<string, NumberWrapper>;
declare class SetNumberWrappersVisitor extends TransformNodesVisitor {
    readonly map: NumberWrapperMap;
    constructor(map: NumberWrapperMap);
}

type StructDefaultValueMap = Record<string, Record<string, StructDefaultValue>>;
type StructDefaultValue = (ValueNode & {
    strategy?: 'optional' | 'omitted';
}) | null;
declare class SetStructDefaultValuesVisitor extends TransformNodesVisitor {
    readonly map: StructDefaultValueMap;
    constructor(map: StructDefaultValueMap);
}

declare class TransformDefinedTypesIntoAccountsVisitor extends BaseNodeVisitor {
    readonly definedTypes: string[];
    constructor(definedTypes: string[]);
    visitProgram(program: ProgramNode): Node;
}

declare class TransformU8ArraysToBytesVisitor extends BaseNodeVisitor {
    readonly sizes: number[] | '*';
    constructor(sizes?: number[] | '*');
    visitArrayType(arrayType: ArrayTypeNode): Node;
    protected hasRequiredSize(size: ArrayTypeNode['size']): boolean;
}

declare class UnwrapDefinedTypesVisitor extends BaseNodeVisitor {
    protected availableDefinedTypes: Map<string, DefinedTypeNode>;
    protected typesToInline: string[] | '*';
    constructor(typesToInline?: string[] | '*');
    visitRoot(root: RootNode): Node;
    visitProgram(program: ProgramNode): Node;
    visitLinkType(linkType: LinkTypeNode): Node;
    protected shouldInline(definedType: string): boolean;
}

declare class UnwrapInstructionArgsDefinedTypesVisitor extends BaseThrowVisitor<RootNode> {
    visitRoot(root: RootNode): RootNode;
}

declare class UnwrapTupleEnumWithSingleStructVisitor extends BaseThrowVisitor<RootNode> {
    readonly enumsOrVariantsToUnwrap: string[] | '*';
    constructor(enumsOrVariantsToUnwrap?: string[] | '*');
    shouldUnwrap(node: EnumTupleVariantTypeNode, stack: NodeStack): boolean;
    visitRoot(root: RootNode): RootNode;
}

declare class UnwrapTypeDefinedLinksVisitor extends TransformNodesVisitor {
    readonly definedLinksType: string[];
    protected readonly availableDefinedTypes: Map<string, DefinedTypeNode>;
    constructor(definedLinksType: string[]);
    visitRoot(root: RootNode): Node | null;
}

type AccountUpdates = NodeTransformer<AccountNode> | {
    delete: true;
} | (Partial<Omit<AccountNodeInput, 'data'>> & {
    data?: Record<string, string>;
});
declare class UpdateAccountsVisitor extends TransformNodesVisitor {
    readonly map: Record<string, AccountUpdates>;
    constructor(map: Record<string, AccountUpdates>);
}

type DefinedTypeUpdates = NodeTransformer<DefinedTypeNode> | {
    delete: true;
} | (Partial<Omit<DefinedTypeNodeInput, 'data'>> & {
    data?: Record<string, string>;
});
declare class UpdateDefinedTypesVisitor extends TransformNodesVisitor {
    readonly map: Record<string, DefinedTypeUpdates>;
    constructor(map: Record<string, DefinedTypeUpdates>);
}

type ErrorUpdates = NodeTransformer<ErrorNode> | {
    delete: true;
} | Partial<ErrorNodeInput>;
declare class UpdateErrorsVisitor extends TransformNodesVisitor {
    readonly map: Record<string, ErrorUpdates>;
    constructor(map: Record<string, ErrorUpdates>);
}

type InstructionUpdates = NodeTransformer<InstructionNode> | {
    delete: true;
} | (InstructionMetadataUpdates & {
    accounts?: InstructionAccountUpdates;
    args?: InstructionArgUpdates;
});
type InstructionMetadataUpdates = Partial<Omit<InstructionNodeInput, 'accounts' | 'dataArgs' | 'extraArgs' | 'subInstructions' | 'argDefaults'>>;
type InstructionAccountUpdates = Record<string, Partial<Omit<InstructionAccountNodeInput, 'defaultsTo'>> & {
    defaultsTo?: InstructionAccountDefault | null;
}>;
type InstructionArgUpdates = Record<string, {
    name?: string;
    docs?: string[];
    type?: TypeNode;
    defaultsTo?: InstructionArgDefault | null;
}>;
declare class UpdateInstructionsVisitor extends TransformNodesVisitor {
    readonly map: Record<string, InstructionUpdates>;
    protected allAccounts: Map<string, AccountNode>;
    constructor(map: Record<string, InstructionUpdates>);
    visitRoot(root: RootNode): Node | null;
    handleInstructionAccount(account: InstructionAccountNode, accountUpdates: InstructionAccountUpdates): InstructionAccountNode;
    handleInstructionArgs(instruction: InstructionNode, newInstructionName: string, argUpdates: InstructionArgUpdates): {
        newDataArgs: InstructionDataArgsNode;
        newExtraArgs: InstructionExtraArgsNode;
        newArgDefaults: Record<string, InstructionArgDefault>;
    };
}

type ProgramUpdates = NodeTransformer<ProgramNode> | {
    delete: true;
} | Partial<Omit<ProgramNodeInput, 'accounts' | 'instructions' | 'definedTypes' | 'errors'>>;
declare class UpdateProgramsVisitor extends TransformNodesVisitor {
    readonly map: Record<string, ProgramUpdates>;
    constructor(map: Record<string, ProgramUpdates>);
}

type CustomAccountSerializerOptions = {
    name: string;
    importFrom: ImportFrom;
    extract: boolean;
    extractAs: string;
    extractedTypeShouldBeInternal: boolean;
};
declare class UseCustomAccountSerializerVisitor extends BaseNodeVisitor {
    readonly map: Record<string, CustomAccountSerializerOptions>;
    constructor(map: Record<string, true | Partial<CustomAccountSerializerOptions>>);
    visitProgram(program: ProgramNode): Node;
    visitAccount(account: AccountNode): Node;
}

type CustomInstructionSerializerOptions = {
    name: string;
    importFrom: ImportFrom;
    extract: boolean;
    extractAs: string;
    extractedTypeShouldBeInternal: boolean;
};
declare class UseCustomInstructionSerializerVisitor extends BaseNodeVisitor {
    readonly map: Record<string, CustomInstructionSerializerOptions>;
    constructor(map: Record<string, true | Partial<CustomInstructionSerializerOptions>>);
    visitProgram(program: ProgramNode): Node;
    visitInstruction(instruction: InstructionNode): Node;
}

declare abstract class BaseDelegateVisitor<T, U> implements Visitor<U> {
    readonly visitor: Visitor<T>;
    constructor(visitor: Visitor<T>);
    abstract map(value: T): U;
    visitRoot(root: RootNode): U;
    visitProgram(program: ProgramNode): U;
    visitAccount(account: AccountNode): U;
    visitAccountData(accountData: AccountDataNode): U;
    visitInstruction(instruction: InstructionNode): U;
    visitInstructionAccount(instructionAccount: InstructionAccountNode): U;
    visitInstructionDataArgs(instructionDataArgs: InstructionDataArgsNode): U;
    visitInstructionExtraArgs(instructionExtraArgs: InstructionExtraArgsNode): U;
    visitDefinedType(definedType: DefinedTypeNode): U;
    visitError(error: ErrorNode): U;
    visitArrayType(arrayType: ArrayTypeNode): U;
    visitLinkType(linkType: LinkTypeNode): U;
    visitEnumType(enumType: EnumTypeNode): U;
    visitEnumEmptyVariantType(enumEmptyVariantType: EnumEmptyVariantTypeNode): U;
    visitEnumStructVariantType(enumStructVariantType: EnumStructVariantTypeNode): U;
    visitEnumTupleVariantType(enumTupleVariantType: EnumTupleVariantTypeNode): U;
    visitMapType(mapType: MapTypeNode): U;
    visitOptionType(optionType: OptionTypeNode): U;
    visitSetType(setType: SetTypeNode): U;
    visitStructType(structType: StructTypeNode): U;
    visitStructFieldType(structFieldType: StructFieldTypeNode): U;
    visitTupleType(tupleType: TupleTypeNode): U;
    visitBoolType(boolType: BoolTypeNode): U;
    visitBytesType(bytesType: BytesTypeNode): U;
    visitNumberType(numberType: NumberTypeNode): U;
    visitNumberWrapperType(numberWrapperType: NumberWrapperTypeNode): U;
    visitPublicKeyType(publicKeyType: PublicKeyTypeNode): U;
    visitStringType(stringType: StringTypeNode): U;
}

declare class ConsoleLogVisitor extends BaseDelegateVisitor<any, void> {
    map(value: any): void;
}

declare class ThrowValidatorItemsVisitor extends BaseDelegateVisitor<ValidatorBag, void> {
    readonly throwLevel: LogLevel;
    constructor(validator: Visitor<ValidatorBag>, throwLevel?: LogLevel);
    map(validatorBag: ValidatorBag): void;
}

declare abstract class BaseVoidVisitor implements Visitor<void> {
    visitRoot(root: RootNode): void;
    visitProgram(program: ProgramNode): void;
    visitAccount(account: AccountNode): void;
    visitAccountData(accountData: AccountDataNode): void;
    visitInstruction(instruction: InstructionNode): void;
    visitInstructionAccount(instructionAccount: InstructionAccountNode): void;
    visitInstructionDataArgs(instructionDataArgs: InstructionDataArgsNode): void;
    visitInstructionExtraArgs(instructionExtraArgs: InstructionExtraArgsNode): void;
    visitDefinedType(definedType: DefinedTypeNode): void;
    visitError(error: ErrorNode): void;
    visitArrayType(arrayType: ArrayTypeNode): void;
    visitLinkType(linkType: LinkTypeNode): void;
    visitEnumType(enumType: EnumTypeNode): void;
    visitEnumEmptyVariantType(enumEmptyVariantType: EnumEmptyVariantTypeNode): void;
    visitEnumStructVariantType(enumStructVariantType: EnumStructVariantTypeNode): void;
    visitEnumTupleVariantType(enumTupleVariantType: EnumTupleVariantTypeNode): void;
    visitMapType(mapType: MapTypeNode): void;
    visitOptionType(optionType: OptionTypeNode): void;
    visitSetType(setType: SetTypeNode): void;
    visitStructType(structType: StructTypeNode): void;
    visitStructFieldType(structFieldType: StructFieldTypeNode): void;
    visitTupleType(tupleType: TupleTypeNode): void;
    visitBoolType(boolType: BoolTypeNode): void;
    visitBytesType(bytesType: BytesTypeNode): void;
    visitNumberType(numberType: NumberTypeNode): void;
    visitNumberWrapperType(numberWrapperType: NumberWrapperTypeNode): void;
    visitPublicKeyType(publicKeyType: PublicKeyTypeNode): void;
    visitStringType(stringType: StringTypeNode): void;
}

declare class DefaultVisitor extends BaseThrowVisitor<RootNode> {
    visitRoot(currentRoot: RootNode): RootNode;
}

type GpaField = {
    name: string;
    offset: number | null;
    type: TypeNode;
};
declare function getGpaFieldsFromAccount(node: AccountNode, sizeVisitor: Visitor<number | null>): GpaField[];

type InstructionDefault = {
    kind: 'identity';
} | {
    kind: 'payer';
} | {
    kind: 'programId';
} | {
    kind: 'program';
    program: {
        name: string;
        publicKey: string;
    };
} | {
    kind: 'publicKey';
    publicKey: string;
} | {
    kind: 'account';
    name: string;
} | {
    kind: 'accountBump';
    name: string;
} | {
    kind: 'arg';
    name: string;
} | {
    kind: 'value';
    value: ValueNode;
} | {
    kind: 'pda';
    pdaAccount: string;
    importFrom: ImportFrom;
    seeds: Record<string, InstructionSeedDefault>;
} | {
    kind: 'resolver';
    name: string;
    importFrom: ImportFrom;
    dependsOn: InstructionDependency[];
    resolvedIsSigner?: boolean | 'either';
    resolvedIsOptional?: boolean;
};
type ExtractInstructionDefault<T extends InstructionDefault['kind']> = Extract<InstructionDefault, {
    kind: T;
}>;
type InstructionAccountDefault = ExtractInstructionDefault<'programId' | 'program' | 'publicKey' | 'account' | 'identity' | 'payer' | 'pda' | 'resolver'>;
type InstructionArgDefault = InstructionDefault;
type InstructionSeedDefault = ExtractInstructionDefault<'arg' | 'account' | 'value'>;
declare const identityDefault: () => ExtractInstructionDefault<'identity'>;
declare const payerDefault: () => ExtractInstructionDefault<'payer'>;
declare const programIdDefault: () => ExtractInstructionDefault<'programId'>;
declare const programDefault: (name: string, publicKey: string) => ExtractInstructionDefault<'program'>;
declare const publicKeyDefault: (publicKey: string) => ExtractInstructionDefault<'publicKey'>;
declare const accountDefault: (account: string) => ExtractInstructionDefault<'account'>;
declare const accountBumpDefault: (account: string) => ExtractInstructionDefault<'accountBump'>;
declare const argDefault: (arg: string) => ExtractInstructionDefault<'arg'>;
declare const valueDefault: (value: ValueNode) => ExtractInstructionDefault<'value'>;
declare const pdaDefault: (pdaAccount: string, options?: {
    importFrom?: ImportFrom;
    seeds?: Record<string, InstructionSeedDefault>;
}) => ExtractInstructionDefault<'pda'>;
declare const resolverDefault: (name: string, dependsOn: InstructionDependency[], options?: {
    importFrom?: ImportFrom;
    resolvedIsSigner?: boolean | 'either';
    resolvedIsOptional?: boolean;
}) => ExtractInstructionDefault<'resolver'>;
type InstructionDependency = {
    kind: 'account' | 'arg';
    name: string;
};
declare const dependsOnAccount: (account: string) => InstructionDependency;
declare const dependsOnArg: (arg: string) => InstructionDependency;
declare const getDefaultSeedsFromAccount: (node: AccountNode) => Record<string, InstructionSeedDefault>;

type IdlType = IdlTypeDefinedLink | IdlTypeOption | IdlTypeTuple | IdlTypeVec | IdlTypeArray | IdlTypeMap | IdlTypeSet | IdlTypeStruct | IdlTypeEnum | IdlTypeLeaf;
type IdlTypeDefinedLink = {
    defined: string;
};
type IdlTypeOption = ({
    option: IdlType;
} | {
    coption: IdlType;
}) & {
    prefix?: IdlTypeUnsignedInteger;
    fixed?: boolean;
};
type IdlTypeTuple = {
    tuple: IdlType[];
};
type IdlTypeVec = {
    vec: IdlType;
    size?: IdlTypeUnsignedInteger | 'remainder';
};
type IdlTypeArray = {
    array: [idlType: IdlType, size: number];
};
type IdlTypeHashMap = {
    hashMap: [IdlType, IdlType];
};
type IdlTypeBTreeMap = {
    bTreeMap: [IdlType, IdlType];
};
type IdlTypeMap = (IdlTypeHashMap | IdlTypeBTreeMap) & {
    size?: IdlTypeUnsignedInteger | number | 'remainder';
};
type IdlTypeHashSet = {
    hashSet: IdlType;
};
type IdlTypeBTreeSet = {
    bTreeSet: IdlType;
};
type IdlTypeSet = (IdlTypeHashSet | IdlTypeBTreeSet) & {
    size?: IdlTypeUnsignedInteger | number | 'remainder';
};
type IdlTypeStruct = {
    kind: 'struct';
    fields: IdlTypeStructField[];
};
type IdlTypeStructField = {
    name: string;
    type: IdlType;
    docs?: string[];
    defaultsValue?: any;
};
type IdlTypeEnum = {
    kind: 'enum';
    variants: IdlTypeEnumVariant[];
    size?: IdlTypeUnsignedInteger;
};
type IdlTypeEnumVariant = {
    name: string;
    fields?: IdlTypeEnumFields;
};
type IdlTypeEnumFields = IdlTypeEnumField[] | IdlType[];
type IdlTypeEnumField = {
    name: string;
    type: IdlType;
    docs?: string[];
};
declare const IDL_TYPE_LEAVES: readonly ["string", "publicKey", "bytes", "bool", "u8", "u16", "u32", "u64", "u128", "i8", "i16", "i32", "i64", "i128", "f32", "f64"];
type IdlTypeLeaf = typeof IDL_TYPE_LEAVES[number];
type IdlTypeUnsignedInteger = 'u8' | 'u16' | 'u32' | 'u64' | 'u128';
type IdlTypeSignedInteger = 'i8' | 'i16' | 'i32' | 'i64' | 'i128';
type IdlTypeInteger = IdlTypeUnsignedInteger | IdlTypeSignedInteger;
type IdlTypeDecimals = 'f32' | 'f64';
type IdlTypeNumber = IdlTypeInteger | IdlTypeDecimals;

type IdlAccount = {
    name: string;
    type: IdlTypeStruct;
    docs?: string[];
    seeds?: IdlAccountSeed[];
    size?: number;
};
type IdlAccountSeed = {
    kind: 'programId';
} | {
    kind: 'constant';
    type: IdlTypeLeaf;
    value: string | boolean | number;
} | {
    kind: 'variable';
    name: string;
    description: string;
    type: IdlTypeLeaf;
};

type IdlDefinedType = {
    name: string;
    type: IdlTypeStruct | IdlTypeEnum;
    docs?: string[];
};

type IdlError = {
    code?: number;
    name?: string;
    msg?: string;
    docs?: string[];
};

type IdlEvent = {
    name: string;
    fields: IdlEventField[];
};
type IdlEventField = {
    name: string;
    type: IdlType;
    index: boolean;
};

type IdlInstruction = {
    name: string;
    accounts: IdlInstructionAccount[];
    args: IdlInstructionArg[];
    defaultOptionalAccounts?: boolean;
    legacyOptionalAccountsStrategy?: boolean;
    discriminant?: IdlInstructionDiscriminant;
    docs?: string[];
};
type IdlInstructionAccount = {
    name: string;
    isMut: boolean;
    isSigner: boolean;
    isOptionalSigner?: boolean;
    isOptional?: boolean;
    optional?: boolean;
    desc?: string;
};
type IdlInstructionArg = {
    name: string;
    type: IdlType;
};
type IdlInstructionDiscriminant = {
    type: IdlType;
    value: number;
};

type Idl = {
    version: string;
    name: string;
    instructions: IdlInstruction[];
    accounts?: IdlAccount[];
    errors?: IdlError[];
    types?: IdlDefinedType[];
    events?: IdlEvent[];
    metadata: {
        address: string;
        origin?: 'anchor' | 'shank';
        binaryVersion?: string;
        libVersion?: string;
    };
};

type AccountNode = {
    readonly __accountNode: unique symbol;
    readonly kind: 'accountNode';
    readonly name: string;
    readonly data: AccountDataNode;
    readonly idlName: string;
    readonly docs: string[];
    readonly internal: boolean;
    readonly size?: number;
    readonly seeds: AccountSeed[];
    readonly discriminator?: AccountDiscriminator;
};
type AccountNodeInput = Omit<PartialExcept<AccountNode, 'name' | 'data'>, '__accountNode' | 'kind'>;
declare function accountNode(input: AccountNodeInput): AccountNode;
declare function accountNodeFromIdl(idl: Partial<IdlAccount>): AccountNode;
declare function isAccountNode(node: Node | null): node is AccountNode;
declare function assertAccountNode(node: Node | null): asserts node is AccountNode;

type ArrayTypeNode = {
    readonly __arrayTypeNode: unique symbol;
    readonly kind: 'arrayTypeNode';
    readonly child: TypeNode;
    readonly size: SizeStrategy;
};
declare function arrayTypeNode(child: TypeNode, options?: {
    readonly size?: ArrayTypeNode['size'];
}): ArrayTypeNode;
declare function arrayTypeNodeFromIdl(idl: IdlTypeArray | IdlTypeVec): ArrayTypeNode;
declare function isArrayTypeNode(node: Node | null): node is ArrayTypeNode;
declare function assertArrayTypeNode(node: Node | null): asserts node is ArrayTypeNode;

type BoolTypeNode = {
    readonly __boolTypeNode: unique symbol;
    readonly kind: 'boolTypeNode';
    readonly size: NumberTypeNode;
};
declare function boolTypeNode(size?: NumberTypeNode): BoolTypeNode;
declare function displayBoolTypeNode(node: BoolTypeNode): string;
declare function isBoolTypeNode(node: Node | null): node is BoolTypeNode;
declare function assertBoolTypeNode(node: Node | null): asserts node is BoolTypeNode;

type BytesTypeNode = {
    readonly __bytesTypeNode: unique symbol;
    readonly kind: 'bytesTypeNode';
    readonly size: SizeStrategy;
};
declare function bytesTypeNode(size?: SizeStrategy): BytesTypeNode;
declare function displayBytesTypeNode(node: BytesTypeNode): string;
declare function isBytesTypeNode(node: Node | null): node is BytesTypeNode;
declare function assertBytesTypeNode(node: Node | null): asserts node is BytesTypeNode;

type EnumEmptyVariantTypeNode = {
    readonly __enumEmptyVariantTypeNode: unique symbol;
    readonly kind: 'enumEmptyVariantTypeNode';
    readonly name: string;
};
declare function enumEmptyVariantTypeNode(name: string): EnumEmptyVariantTypeNode;
declare function enumEmptyVariantTypeNodeFromIdl(idl: IdlTypeEnumVariant): EnumEmptyVariantTypeNode;
declare function isEnumEmptyVariantTypeNode(node: Node | null): node is EnumEmptyVariantTypeNode;
declare function assertEnumEmptyVariantTypeNode(node: Node | null): asserts node is EnumEmptyVariantTypeNode;

type StructFieldTypeNode = {
    readonly __structFieldTypeNode: unique symbol;
    readonly kind: 'structFieldTypeNode';
    readonly name: string;
    readonly child: TypeNode;
    readonly docs: string[];
    readonly defaultsTo: {
        strategy: 'optional' | 'omitted';
        value: ValueNode;
    } | null;
};
type StructFieldTypeNodeInput = {
    readonly name: string;
    readonly child: TypeNode;
    readonly docs?: string[];
    readonly defaultsTo?: StructFieldTypeNode['defaultsTo'];
};
declare function structFieldTypeNode(input: StructFieldTypeNodeInput): StructFieldTypeNode;
declare function structFieldTypeNodeFromIdl(idl: IdlTypeStructField): StructFieldTypeNode;
declare function isStructFieldTypeNode(node: Node | null): node is StructFieldTypeNode;
declare function assertStructFieldTypeNode(node: Node | null): asserts node is StructFieldTypeNode;

type StructTypeNode = {
    readonly __structTypeNode: unique symbol;
    readonly kind: 'structTypeNode';
    readonly fields: StructFieldTypeNode[];
};
declare function structTypeNode(fields: StructFieldTypeNode[]): StructTypeNode;
declare function structTypeNodeFromIdl(idl: IdlTypeStruct): StructTypeNode;
declare function isStructTypeNode(node: Node | null): node is StructTypeNode;
declare function assertStructTypeNode(node: Node | null): asserts node is StructTypeNode;

type EnumStructVariantTypeNode = {
    readonly __enumStructVariantTypeNode: unique symbol;
    readonly kind: 'enumStructVariantTypeNode';
    readonly name: string;
    readonly struct: StructTypeNode;
};
declare function enumStructVariantTypeNode(name: string, struct: StructTypeNode): EnumStructVariantTypeNode;
declare function enumStructVariantTypeNodeFromIdl(idl: IdlTypeEnumVariant): EnumStructVariantTypeNode;
declare function isEnumStructVariantTypeNode(node: Node | null): node is EnumStructVariantTypeNode;
declare function assertEnumStructVariantTypeNode(node: Node | null): asserts node is EnumStructVariantTypeNode;

type TupleTypeNode = {
    readonly __tupleTypeNode: unique symbol;
    readonly kind: 'tupleTypeNode';
    readonly children: TypeNode[];
};
declare function tupleTypeNode(children: TypeNode[]): TupleTypeNode;
declare function tupleTypeNodeFromIdl(idl: IdlTypeTuple): TupleTypeNode;
declare function isTupleTypeNode(node: Node | null): node is TupleTypeNode;
declare function assertTupleTypeNode(node: Node | null): asserts node is TupleTypeNode;

type EnumTupleVariantTypeNode = {
    readonly __enumTupleVariantTypeNode: unique symbol;
    readonly kind: 'enumTupleVariantTypeNode';
    readonly name: string;
    readonly tuple: TupleTypeNode;
};
declare function enumTupleVariantTypeNode(name: string, tuple: TupleTypeNode): EnumTupleVariantTypeNode;
declare function enumTupleVariantTypeNodeFromIdl(idl: IdlTypeEnumVariant): EnumTupleVariantTypeNode;
declare function isEnumTupleVariantTypeNode(node: Node | null): node is EnumTupleVariantTypeNode;
declare function assertEnumTupleVariantTypeNode(node: Node | null): asserts node is EnumTupleVariantTypeNode;

type EnumVariantTypeNode = EnumEmptyVariantTypeNode | EnumStructVariantTypeNode | EnumTupleVariantTypeNode;
declare function isEnumVariantTypeNode(node: Node | null): node is EnumVariantTypeNode;
declare function assertEnumVariantTypeNode(node: Node | null): asserts node is EnumVariantTypeNode;

type EnumTypeNode = {
    readonly __enumTypeNode: unique symbol;
    readonly kind: 'enumTypeNode';
    readonly variants: EnumVariantTypeNode[];
    readonly size: NumberTypeNode;
};
declare function enumTypeNode(variants: EnumVariantTypeNode[], options?: {
    size?: NumberTypeNode;
}): EnumTypeNode;
declare function enumTypeNodeFromIdl(idl: IdlTypeEnum): EnumTypeNode;
declare function isScalarEnum(node: EnumTypeNode): boolean;
declare function isDataEnum(node: EnumTypeNode): boolean;
declare function isEnumTypeNode(node: Node | null): node is EnumTypeNode;
declare function assertEnumTypeNode(node: Node | null): asserts node is EnumTypeNode;

type MapTypeNode = {
    readonly __mapTypeNode: unique symbol;
    readonly kind: 'mapTypeNode';
    readonly key: TypeNode;
    readonly value: TypeNode;
    readonly size: SizeStrategy;
    readonly idlMap: 'hashMap' | 'bTreeMap';
};
declare function mapTypeNode(key: TypeNode, value: TypeNode, options?: {
    readonly size?: SizeStrategy;
    readonly idlMap?: MapTypeNode['idlMap'];
}): MapTypeNode;
declare function mapTypeNodeFromIdl(idl: IdlTypeMap): MapTypeNode;
declare function isMapTypeNode(node: Node | null): node is MapTypeNode;
declare function assertMapTypeNode(node: Node | null): asserts node is MapTypeNode;

type NumberWrapper = {
    kind: 'DateTime';
} | {
    kind: 'SolAmount';
} | {
    kind: 'Amount';
    identifier: string;
    decimals: number;
};
type NumberWrapperTypeNode = {
    readonly __numberWrapperTypeNode: unique symbol;
    readonly kind: 'numberWrapperTypeNode';
    readonly number: NumberTypeNode;
    readonly wrapper: NumberWrapper;
};
declare function numberWrapperTypeNode(number: NumberTypeNode, wrapper: NumberWrapper): NumberWrapperTypeNode;
declare function isNumberWrapperTypeNode(node: Node | null): node is NumberWrapperTypeNode;
declare function assertNumberWrapperTypeNode(node: Node | null): asserts node is NumberWrapperTypeNode;

type OptionTypeNode = {
    readonly __optionTypeNode: unique symbol;
    readonly kind: 'optionTypeNode';
    readonly child: TypeNode;
    readonly prefix: NumberTypeNode;
    readonly fixed: boolean;
    readonly idlOption: 'option' | 'coption';
};
declare function optionTypeNode(child: TypeNode, options?: {
    readonly prefix?: NumberTypeNode;
    readonly fixed?: boolean;
    readonly idlOption?: OptionTypeNode['idlOption'];
}): OptionTypeNode;
declare function optionTypeNodeFromIdl(idl: IdlTypeOption): OptionTypeNode;
declare function isOptionTypeNode(node: Node | null): node is OptionTypeNode;
declare function assertOptionTypeNode(node: Node | null): asserts node is OptionTypeNode;

type PublicKeyTypeNode = {
    readonly __publicKeyTypeNode: unique symbol;
    readonly kind: 'publicKeyTypeNode';
};
declare function publicKeyTypeNode(): PublicKeyTypeNode;
declare function isPublicKeyTypeNode(node: Node | null): node is PublicKeyTypeNode;
declare function assertPublicKeyTypeNode(node: Node | null): asserts node is PublicKeyTypeNode;

type SetTypeNode = {
    readonly __setTypeNode: unique symbol;
    readonly kind: 'setTypeNode';
    readonly child: TypeNode;
    readonly size: SizeStrategy;
    readonly idlSet: 'hashSet' | 'bTreeSet';
};
declare function setTypeNode(child: TypeNode, options?: {
    readonly size?: SizeStrategy;
    readonly idlSet?: SetTypeNode['idlSet'];
}): SetTypeNode;
declare function setTypeNodeFromIdl(idl: IdlTypeSet): SetTypeNode;
declare function isSetTypeNode(node: Node | null): node is SetTypeNode;
declare function assertSetTypeNode(node: Node | null): asserts node is SetTypeNode;

type StringEncoding = 'utf8' | 'base16' | 'base58' | 'base64';
type StringTypeNode = {
    readonly __stringTypeNode: unique symbol;
    readonly kind: 'stringTypeNode';
    readonly encoding: StringEncoding;
    readonly size: SizeStrategy;
};
declare function stringTypeNode(options?: {
    readonly encoding?: StringEncoding;
    readonly size?: SizeStrategy;
}): StringTypeNode;
declare function displayStringTypeNode(node: StringTypeNode): string;
declare function isStringTypeNode(node: Node | null): node is StringTypeNode;
declare function assertStringTypeNode(node: Node | null): asserts node is StringTypeNode;

type TypeNode = ArrayTypeNode | BoolTypeNode | BytesTypeNode | LinkTypeNode | EnumTypeNode | MapTypeNode | NumberTypeNode | NumberWrapperTypeNode | OptionTypeNode | PublicKeyTypeNode | SetTypeNode | StringTypeNode | StructTypeNode | TupleTypeNode;
declare const createTypeNodeFromIdl: (idlType: IdlType) => TypeNode;
declare function isTypeNode(node: Node | null): node is TypeNode;
declare function assertTypeNode(node: Node | null): asserts node is TypeNode;
declare function isStructOrLinkTypeNode(node: Node | null): node is StructTypeNode | LinkTypeNode;
declare function assertStructOrLinkTypeNode(node: Node | null): asserts node is StructTypeNode | LinkTypeNode;

type DefinedTypeNode = {
    readonly __definedTypeNode: unique symbol;
    readonly kind: 'definedTypeNode';
    readonly name: string;
    readonly data: TypeNode;
    readonly idlName: string;
    readonly docs: string[];
    readonly internal: boolean;
};
type DefinedTypeNodeInput = {
    readonly name: string;
    readonly data: TypeNode;
    readonly idlName?: string;
    readonly docs?: string[];
    readonly internal?: boolean;
};
declare function definedTypeNode(input: DefinedTypeNodeInput): DefinedTypeNode;
declare function definedTypeNodeFromIdl(idl: Partial<IdlDefinedType>): DefinedTypeNode;
declare function isDefinedTypeNode(node: Node | null): node is DefinedTypeNode;
declare function assertDefinedTypeNode(node: Node | null): asserts node is DefinedTypeNode;

type ErrorNode = {
    readonly __errorNode: unique symbol;
    readonly kind: 'errorNode';
    readonly name: string;
    readonly idlName: string;
    readonly code: number;
    readonly message: string;
    readonly docs: string[];
};
type ErrorNodeInput = Omit<PartialExcept<ErrorNode, 'name' | 'code' | 'message'>, '__errorNode' | 'kind'>;
declare function errorNode(input: ErrorNodeInput): ErrorNode;
declare function errorNodeFromIdl(idl: Partial<IdlError>): ErrorNode;
declare function isErrorNode(node: Node | null): node is ErrorNode;
declare function assertErrorNode(node: Node | null): asserts node is ErrorNode;

type InstructionDataArgsNode = {
    readonly __instructionDataArgsNode: unique symbol;
    readonly kind: 'instructionDataArgsNode';
    readonly name: string;
    readonly struct: StructTypeNode;
    readonly link?: LinkTypeNode;
};
type InstructionDataArgsNodeInput = Omit<InstructionDataArgsNode, '__instructionDataArgsNode' | 'kind'>;
declare function instructionDataArgsNode(input: InstructionDataArgsNodeInput): InstructionDataArgsNode;
declare function isInstructionDataArgsNode(node: Node | null): node is InstructionDataArgsNode;
declare function assertInstructionDataArgsNode(node: Node | null): asserts node is InstructionDataArgsNode;

type EventNode = {
    readonly __eventNode: unique symbol;
    readonly kind: 'eventNode';
    readonly name: string;
    readonly dataArgs: InstructionDataArgsNode;
};
type EventNodeInput = Omit<PartialExcept<EventNode, 'name' | 'dataArgs'>, '__instructionNode' | 'kind'>;
declare function eventNode(input: EventNodeInput): EventNode;
declare function eventNodeFromIdl(idl: Partial<IdlEvent>): EventNode;
declare function isEventNode(node: Node | null): node is EventNode;
declare function assertEventNode(node: Node | null): asserts node is EventNode;

type InstructionAccountNode = {
    readonly __instructionAccountNode: unique symbol;
    readonly kind: 'instructionAccountNode';
    readonly name: string;
    readonly isWritable: boolean;
    readonly isSigner: boolean | 'either';
    readonly isOptional: boolean;
    readonly docs: string[];
    readonly defaultsTo?: InstructionAccountDefault;
};
type InstructionAccountNodeInput = Omit<PartialExcept<InstructionAccountNode, 'name' | 'isWritable' | 'isSigner'>, '__instructionAccountNode' | 'kind'>;
declare function instructionAccountNode(input: InstructionAccountNodeInput): InstructionAccountNode;
declare function instructionAccountNodeFromIdl(idl: IdlInstructionAccount, useProgramIdForOptionalAccounts?: boolean): InstructionAccountNode;
declare function isInstructionAccountNode(node: Node | null): node is InstructionAccountNode;
declare function assertInstructionAccountNode(node: Node | null): asserts node is InstructionAccountNode;

type InstructionExtraArgsNode = {
    readonly __instructionExtraArgsNode: unique symbol;
    readonly kind: 'instructionExtraArgsNode';
    readonly name: string;
    readonly struct: StructTypeNode;
    readonly link?: LinkTypeNode;
};
type InstructionExtraArgsNodeInput = Omit<InstructionExtraArgsNode, '__instructionExtraArgsNode' | 'kind'>;
declare function instructionExtraArgsNode(input: InstructionExtraArgsNodeInput): InstructionExtraArgsNode;
declare function isInstructionExtraArgsNode(node: Node | null): node is InstructionExtraArgsNode;
declare function assertInstructionExtraArgsNode(node: Node | null): asserts node is InstructionExtraArgsNode;

type ProgramNode = {
    readonly __programNode: unique symbol;
    readonly kind: 'programNode';
    readonly accounts: AccountNode[];
    readonly instructions: InstructionNode[];
    readonly definedTypes: DefinedTypeNode[];
    readonly errors: ErrorNode[];
    readonly name: string;
    readonly prefix: string;
    readonly publicKey: string;
    readonly version: string;
    readonly events?: EventNode[];
    readonly origin?: 'shank' | 'anchor';
    readonly internal: boolean;
};
type ProgramNodeInput = Omit<PartialExcept<ProgramNode, 'accounts' | 'instructions' | 'definedTypes' | 'errors' | 'name' | 'publicKey' | 'version'>, '__programNode' | 'kind'>;
declare function programNode(input: ProgramNodeInput): ProgramNode;
declare function programNodeFromIdl(idl: Partial<Idl>): ProgramNode;
declare function isProgramNode(node: Node | null): node is ProgramNode;
declare function assertProgramNode(node: Node | null): asserts node is ProgramNode;

type IdlInputs = string | Partial<Idl> | (string | Partial<Idl>)[];
type RootNode = {
    readonly __rootNode: unique symbol;
    readonly kind: 'rootNode';
    readonly programs: ProgramNode[];
};
declare function rootNode(programs: ProgramNode[]): RootNode;
declare function rootNodeFromIdls(idls: IdlInputs): RootNode;
declare function getAllAccounts(node: RootNode): AccountNode[];
declare function getAllDefinedTypes(node: RootNode): DefinedTypeNode[];
declare function getAllInstructions(node: RootNode): InstructionNode[];
declare function getAllErrors(node: RootNode): ErrorNode[];
declare function getAllEvents(node: RootNode): EventNode[];
declare function isRootNode(node: Node | null): node is RootNode;
declare function assertRootNode(node: Node | null): asserts node is RootNode;

type InstructionNode = {
    readonly __instructionNode: unique symbol;
    readonly kind: 'instructionNode';
    readonly name: string;
    readonly accounts: InstructionAccountNode[];
    readonly dataArgs: InstructionDataArgsNode;
    readonly extraArgs: InstructionExtraArgsNode;
    readonly subInstructions: InstructionNode[];
    readonly idlName: string;
    readonly docs: string[];
    readonly internal: boolean;
    readonly bytesCreatedOnChain?: BytesCreatedOnChain;
    readonly argDefaults: Record<string, InstructionArgDefault>;
};
type InstructionNodeInput = Omit<PartialExcept<InstructionNode, 'name' | 'accounts' | 'dataArgs'>, '__instructionNode' | 'kind'>;
declare function instructionNode(input: InstructionNodeInput): InstructionNode;
declare function instructionNodeFromIdl(idl: Partial<IdlInstruction>): InstructionNode;
declare function getAllInstructionsWithSubs(node: ProgramNode | RootNode | InstructionNode, leavesOnly?: boolean): InstructionNode[];
declare function isInstructionNode(node: Node | null): node is InstructionNode;
declare function assertInstructionNode(node: Node | null): asserts node is InstructionNode;

type Node = RootNode | ProgramNode | AccountNode | AccountDataNode | InstructionNode | InstructionAccountNode | InstructionDataArgsNode | InstructionExtraArgsNode | ErrorNode | EventNode | DefinedTypeNode | TypeNode | StructFieldTypeNode | EnumVariantTypeNode;
declare const assertNodeFilter: <T extends Node>(assertCallback: (node: Node | null) => asserts node is T) => (node: Node | null | null) => node is T;
declare const removeNullAndAssertNodeFilter: <T extends Node>(assertCallback: (node: Node | null) => asserts node is T) => (node: Node | null | null) => node is T;

type NumberFormat = 'u8' | 'u16' | 'u32' | 'u64' | 'u128' | 'i8' | 'i16' | 'i32' | 'i64' | 'i128' | 'f32' | 'f64';
type NumberTypeNode = {
    readonly __numberTypeNode: unique symbol;
    readonly kind: 'numberTypeNode';
    readonly format: NumberFormat;
    readonly endian: 'le' | 'be';
};
declare function numberTypeNode(format: NumberFormat, endian?: 'le' | 'be'): NumberTypeNode;
declare function isSignedInteger(node: NumberTypeNode): boolean;
declare function isUnsignedInteger(node: NumberTypeNode): boolean;
declare function isInteger(node: NumberTypeNode): boolean;
declare function isDecimal(node: NumberTypeNode): boolean;
declare function displayNumberTypeNode(node: NumberTypeNode): string;
declare function isNumberTypeNode(node: Node | null): node is NumberTypeNode;
declare function assertNumberTypeNode(node: Node | null): asserts node is NumberTypeNode;

type SizeStrategy = {
    kind: 'fixed';
    value: number;
} | {
    kind: 'prefixed';
    prefix: NumberTypeNode;
} | {
    kind: 'remainder';
};
declare const fixedSize: (value: number) => SizeStrategy;
declare const prefixedSize: (prefix?: NumberTypeNode) => SizeStrategy;
declare const remainderSize: () => SizeStrategy;
declare const displaySizeStrategy: (size: SizeStrategy) => string;

declare class KinobiError extends Error {
    readonly name: string;
}
declare class InvalidKinobiTreeError extends KinobiError {
    readonly name: string;
}

type PickPartial<T, K extends keyof T> = Omit<T, K> & Partial<Pick<T, K>>;
type PartialExcept<T, K extends keyof T> = Pick<T, K> & Partial<Omit<T, K>>;
declare function capitalize(str: string): string;
declare function titleCase(str: string): string;
declare function pascalCase(str: string): string;
declare function camelCase(str: string): string;
declare function kebabCase(str: string): string;
declare function snakeCase(str: string): string;
declare function mainCase(str: string): string;

type LinkTypeNode = {
    readonly __linkTypeNode: unique symbol;
    readonly kind: 'linkTypeNode';
    readonly name: string;
    readonly importFrom: ImportFrom;
    readonly size?: number;
};
declare function linkTypeNode(name: string, options?: {
    readonly importFrom?: ImportFrom;
    readonly size?: number;
}): LinkTypeNode;
declare function isLinkTypeNode(node: Node | null): node is LinkTypeNode;
declare function assertLinkTypeNode(node: Node | null): asserts node is LinkTypeNode;

type AccountDataNode = {
    readonly __accountDataNode: unique symbol;
    readonly kind: 'accountDataNode';
    readonly name: string;
    readonly struct: StructTypeNode;
    readonly link?: LinkTypeNode;
};
type AccountDataNodeInput = Omit<AccountDataNode, '__accountDataNode' | 'kind'>;
declare function accountDataNode(input: AccountDataNodeInput): AccountDataNode;
declare function isAccountDataNode(node: Node | null): node is AccountDataNode;
declare function assertAccountDataNode(node: Node | null): asserts node is AccountDataNode;

interface Kinobi {
    getRoot(): RootNode;
    getJson(): string;
    accept<T>(visitor: Visitor<T>): T;
    update(visitor: Visitor<Node | null>): void;
    clone(): Kinobi;
}
declare function createFromRoot(root: RootNode, useDefaultVisitor?: boolean): Kinobi;
declare function createFromIdls(idls: IdlInputs, useDefaultVisitor?: boolean): Kinobi;
declare function createFromJson(json: string, useDefaultVisitor?: boolean): Kinobi;

export { AccountDataNode, AccountDataNodeInput, AccountDiscriminator, AccountNode, AccountNodeInput, AccountSeed, AccountUpdates, AnchorDiscriminator, ArrayTypeNode, AutoSetAnchorDiscriminatorsVisitor, AutoSetFixedAccountSizesVisitor, BaseDelegateVisitor, BaseNodeOrNullVisitor, BaseNodeVisitor, BaseThrowVisitor, BaseVoidVisitor, BoolTypeNode, BytesCreatedOnChain, BytesTypeNode, ConsoleLogVisitor, CreateSubInstructionsFromEnumArgsVisitor, CustomAccountSerializerOptions, CustomInstructionSerializerOptions, DEFAULT_INSTRUCTION_ACCOUNT_DEFAULT_RULES, DeduplicateIdenticalDefinedTypesVisitor, DefaultVisitor, DefinedTypeHistogram, DefinedTypeNode, DefinedTypeNodeInput, DefinedTypeUpdates, DeleteNodesVisitor, EnumEmptyVariantTypeNode, EnumStructVariantTypeNode, EnumTupleVariantTypeNode, EnumTypeNode, EnumValueNode, EnumVariantTypeNode, ErrorNode, ErrorNodeInput, ErrorUpdates, EventNode, EventNodeInput, ExtractInstructionDefault, FlattenInstructionArgsStructVisitor, FlattenStructOptions, FlattenStructVisitor, GetByteSizeVisitor, GetDefaultValidatorBagVisitor, GetDefinedTypeHistogramVisitor, GetNodeInlineStringVisitor, GetNodeTreeStringVisitor, GetResolvedInstructionInputsVisitor, GpaField, IDL_TYPE_LEAVES, Idl, IdlAccount, IdlAccountSeed, IdlDefinedType, IdlError, IdlEvent, IdlEventField, IdlInputs, IdlInstruction, IdlInstructionAccount, IdlInstructionArg, IdlInstructionDiscriminant, IdlType, IdlTypeArray, IdlTypeBTreeMap, IdlTypeBTreeSet, IdlTypeDecimals, IdlTypeDefinedLink, IdlTypeEnum, IdlTypeEnumField, IdlTypeEnumFields, IdlTypeEnumVariant, IdlTypeHashMap, IdlTypeHashSet, IdlTypeInteger, IdlTypeLeaf, IdlTypeMap, IdlTypeNumber, IdlTypeOption, IdlTypeSet, IdlTypeSignedInteger, IdlTypeStruct, IdlTypeStructField, IdlTypeTuple, IdlTypeUnsignedInteger, IdlTypeVec, ImportFrom, InstructionAccountDefault, InstructionAccountDefaultRule, InstructionAccountNode, InstructionAccountNodeInput, InstructionAccountUpdates, InstructionArgDefault, InstructionArgUpdates, InstructionDataArgsNode, InstructionDataArgsNodeInput, InstructionDefault, InstructionDependency, InstructionExtraArgsNode, InstructionExtraArgsNodeInput, InstructionMetadataUpdates, InstructionNode, InstructionNodeInput, InstructionSeedDefault, InstructionUpdates, InvalidKinobiTreeError, Kinobi, KinobiError, LOG_LEVELS, LinkTypeNode, ListValueNode, LogLevel, MapTypeNode, MapValueNode, Node, NodeSelector, NodeSelectorFunction, NodeStack, NodeTransform, NodeTransformer, NumberFormat, NumberTypeNode, NumberWrapper, NumberWrapperTypeNode, OptionTypeNode, OptionValueNode, PartialExcept, PickPartial, ProgramNode, ProgramNodeInput, ProgramUpdates, PublicKeyTypeNode, PublicKeyValueNode, RelativeImportFrom, ResolvedInstructionAccount, ResolvedInstructionArg, ResolvedInstructionInput, RootNode, ScalarValueNode, SetAccountDiscriminatorFromFieldVisitor, SetInstructionAccountDefaultValuesVisitor, SetInstructionDiscriminatorsVisitor, SetMissingDefinedTypesVisitor, SetNumberWrappersVisitor, SetStructDefaultValuesVisitor, SetTypeNode, SetValueNode, SizeStrategy, StringEncoding, StringTypeNode, StructFieldTypeNode, StructFieldTypeNodeInput, StructTypeNode, StructValueNode, ThrowValidatorItemsVisitor, TransformDefinedTypesIntoAccountsVisitor, TransformNodesVisitor, TransformU8ArraysToBytesVisitor, TupleTypeNode, TupleValueNode, TypeNode, UnwrapDefinedTypesVisitor, UnwrapInstructionArgsDefinedTypesVisitor, UnwrapTupleEnumWithSingleStructVisitor, UnwrapTypeDefinedLinksVisitor, UpdateAccountsVisitor, UpdateDefinedTypesVisitor, UpdateErrorsVisitor, UpdateInstructionsVisitor, UpdateProgramsVisitor, UseCustomAccountSerializerVisitor, UseCustomInstructionSerializerVisitor, ValidatorBag, ValidatorItem, ValueNode, Visitor, accountBumpDefault, accountDataNode, accountDefault, accountNode, accountNodeFromIdl, argDefault, arrayTypeNode, arrayTypeNodeFromIdl, assertAccountDataNode, assertAccountNode, assertArrayTypeNode, assertBoolTypeNode, assertBytesTypeNode, assertDefinedTypeNode, assertEnumEmptyVariantTypeNode, assertEnumStructVariantTypeNode, assertEnumTupleVariantTypeNode, assertEnumTypeNode, assertEnumVariantTypeNode, assertErrorNode, assertEventNode, assertInstructionAccountNode, assertInstructionDataArgsNode, assertInstructionExtraArgsNode, assertInstructionNode, assertLinkTypeNode, assertMapTypeNode, assertNodeFilter, assertNumberTypeNode, assertNumberWrapperTypeNode, assertOptionTypeNode, assertProgramNode, assertPublicKeyTypeNode, assertRootNode, assertSetTypeNode, assertStringTypeNode, assertStructFieldTypeNode, assertStructOrLinkTypeNode, assertStructTypeNode, assertTupleTypeNode, assertTypeNode, boolTypeNode, bytesFromAccount, bytesFromArg, bytesFromNumber, bytesFromResolver, bytesTypeNode, camelCase, capitalize, constantSeed, createFromIdls, createFromJson, createFromRoot, createTypeNodeFromIdl, definedTypeNode, definedTypeNodeFromIdl, dependsOnAccount, dependsOnArg, displayBoolTypeNode, displayBytesTypeNode, displayNumberTypeNode, displaySizeStrategy, displayStringTypeNode, enumEmptyVariantTypeNode, enumEmptyVariantTypeNodeFromIdl, enumStructVariantTypeNode, enumStructVariantTypeNodeFromIdl, enumTupleVariantTypeNode, enumTupleVariantTypeNodeFromIdl, enumTypeNode, enumTypeNodeFromIdl, errorNode, errorNodeFromIdl, eventNode, eventNodeFromIdl, fieldAccountDiscriminator, fixedSize, flattenStruct, getAllAccounts, getAllDefinedTypes, getAllErrors, getAllEvents, getAllInstructions, getAllInstructionsWithSubs, getAnchorAccountDiscriminator, getAnchorInstructionDiscriminator, getDefaultSeedsFromAccount, getGpaFieldsFromAccount, getLevelIndex, identityDefault, instructionAccountNode, instructionAccountNodeFromIdl, instructionDataArgsNode, instructionExtraArgsNode, instructionNode, instructionNodeFromIdl, isAccountDataNode, isAccountNode, isArrayTypeNode, isBoolTypeNode, isBytesTypeNode, isDataEnum, isDecimal, isDefinedTypeNode, isEnumEmptyVariantTypeNode, isEnumStructVariantTypeNode, isEnumTupleVariantTypeNode, isEnumTypeNode, isEnumVariantTypeNode, isErrorNode, isEventNode, isInstructionAccountNode, isInstructionDataArgsNode, isInstructionExtraArgsNode, isInstructionNode, isInteger, isLinkTypeNode, isMapTypeNode, isNumberTypeNode, isNumberWrapperTypeNode, isOptionTypeNode, isProgramNode, isPublicKeyTypeNode, isRootNode, isScalarEnum, isSetTypeNode, isSignedInteger, isStringTypeNode, isStructFieldTypeNode, isStructOrLinkTypeNode, isStructTypeNode, isTupleTypeNode, isTypeNode, isUnsignedInteger, kebabCase, linkTypeNode, logDebug, logError, logInfo, logTrace, logWarn, mainCase, mapTypeNode, mapTypeNodeFromIdl, numberTypeNode, numberWrapperTypeNode, optionTypeNode, optionTypeNodeFromIdl, pascalCase, payerDefault, pdaDefault, prefixedSize, programDefault, programIdDefault, programNode, programNodeFromIdl, programSeed, publicKeyDefault, publicKeySeed, publicKeyTypeNode, remainderSize, removeNullAndAssertNodeFilter, resolverDefault, rootNode, rootNodeFromIdls, setTypeNode, setTypeNodeFromIdl, shouldLogLevel, sizeAccountDiscriminator, snakeCase, stringConstantSeed, stringSeed, stringTypeNode, structFieldTypeNode, structFieldTypeNodeFromIdl, structTypeNode, structTypeNodeFromIdl, titleCase, toNodeSelectorFunction, tupleTypeNode, tupleTypeNodeFromIdl, vEnum, vList, vMap, vNone, vPublicKey, vScalar, vSet, vSome, vStruct, vTuple, valueDefault, variableSeed, visit };
